/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.util.*;

/**
 * A GUI object to hold a number of shapes.  Helps isolate the code
 * for drag and drop.
 *
 * @author Neil Birkbeck
 */
public abstract class BasicShapePanel extends JPanel
{
    class Listener implements DragSourceListener {
	public void dragDropEnd(DragSourceDropEvent dsde){ }

	public void dragEnter(DragSourceDragEvent e){
	    DragSourceContext context = e.getDragSourceContext();
	    int myaction = e.getDropAction();
	    if( (myaction & DnDConstants.ACTION_COPY) != 0) { 
		context.setCursor(DragSource.DefaultCopyDrop);    
	    } 
	    else if ((myaction & DnDConstants.ACTION_MOVE) !=0){
		context.setCursor(DragSource.DefaultMoveDrop);
	    }
	    else {
		context.setCursor(DragSource.DefaultCopyNoDrop);        
	    }
	}

	public void dragExit(DragSourceEvent dse){
	    System.out.println("BasicShapePanel: Drag Exit");
	}
	
	public void dragOver(DragSourceDragEvent dse){
	    System.out.println("BasicShapePanel: Drag Over");
	}

	public void dropActionChanged(DragSourceDragEvent dse){
	    System.out.println("BasicShapePanel: Drop Action Changed\n");
	}
    };

    BasicShapePanel(){ 
	this.dragSource = DragSource.getDefaultDragSource();
	this.dsListener = new Listener();
	this.dgListener = new DragGestureListener(){
		public void dragGestureRecognized(DragGestureEvent dge){
		    System.out.println("BasicShapePanel::DragGesture Recognized\n");
		    
		    // check to see if action is OK ...
		    try {
			Transferable transferable = 
			    new ShapeTransferable(getShapes(), (int)getCanvasWidth(), (int)getCanvasHeight());//StringSelection("Hello\n");
			    
			//initial cursor, transferable, dsource listener      
			dragSource.startDrag(dge, DragSource.DefaultCopyNoDrop, transferable, dsListener);
			    
		    }catch( InvalidDnDOperationException idoe ) {
			System.err.println( idoe );
		    }
		}
	    };
	this.dragSource.createDefaultDragGestureRecognizer(
							   this, DnDConstants.ACTION_COPY_OR_MOVE, this.dgListener );
    }
    public abstract int getCanvasWidth();
    public abstract int getCanvasHeight();
    public abstract Vector<Shape> getShapes();

    private DragSource dragSource;
    private DragGestureListener dgListener;
    private Listener dsListener;
}
