/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

/**
 * A class to represent a bezier triple.
 *
 * @author Neil Birkbeck.
 */
public class BezTriple {

    public Point2D   pt;
    public Point2D prev;
    public Point2D next;

    public boolean  line;
	
    /**
     * Create a new bezier triple with the point p,
     * the previous, pr, and the next ne.
     *
     * @param p the point
     * @param pr the previous handle point.
     * @param ne the next handle point.
     */
    BezTriple(Point2D p, Point2D pr, Point2D ne){
	line = false;
	pt = p;
	prev = pr;
	next = ne;
    }
	
    /**
     * Subtract the point from this.
     * 
     * @param p the point to translate by.
     * @fixme  This subtract moves the object in place.
     */
    void subtract(Point2D p){
	pt = pt.subtract(p);
	prev = prev.subtract(p);
	next = next.subtract(p);
    }
	
    /**
     * Add the point to this.
     * 
     * @param p the point to translate by.
     * @fixme  This moves the object in place.
     */
    void add(Point2D p){
	pt = pt.add(p);
	prev = prev.add(p);
	next = next.add(p);
    }

    /**
     * Scale the point (and handles).
     * 
     * @param d the amount to scale.
     * @fixme  This scales the object in place.
     */
    void scale(double d){
	pt = pt.scale(d);
	prev = prev.scale(d);
	next = next.scale(d);
    }

}

