/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.util.*;

/**
 * The transferable representation of a shape parameters
 * used in drag-and-drop operations.
 */
public class ParamTransferable implements Transferable {
    String params;

    /**
     * Create new transferable object with the given string.
     *
     * @param str the string representation.
     */
    public ParamTransferable(String str){
	params = str;
    }
    
    public Object getTransferData(DataFlavor flavor){
	return new ByteArrayInputStream(params.getBytes());
    }

    /**
     * The only supported data flavour is MimeType text/plain.
     */	
    public DataFlavor[] getTransferDataFlavors(){
	DataFlavor [] flav = new DataFlavor[1];
	try {
	    flav[0] = new DataFlavor("text/plain");//image/jpg");
	}
	catch(Exception e){
	    System.out.println("Caught exception in ParamTransferable: "+e);
	}
	return flav;
    }

    public boolean isDataFlavorSupported(DataFlavor flavor){
	// System.out.println("Data supported "+flavor.getMimeType());
	return false;
    }
};