/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

/**
 * An extension of the java Point2D.Double with functionality
 * to ease mathematical computations (addition, dot-product, etc.)
 */
class Point2D extends java.awt.geom.Point2D.Double {

    Point2D() {
	super();
    }

    Point2D(double x, double y) {
	super(x, y);
    }

    /**
     * @return this - v
     */
    public Point2D subtract(Point2D v) {
	return new Point2D(getX() - v.getX(), getY() - v.getY());
    }

    /**
     * @return this + v
     */
    public Point2D add(Point2D v) {
	return new Point2D(getX() + v.getX(), getY() + v.getY());
    }

    /**
     * @return this * v
     */
    public Point2D scale(double v) {
	return new Point2D(getX()*v, getY()*v);
    }

    /**
     * Rotate the point.
     *
     * @param t  the angle (in radians) to rotate.
     * @return the rotated point
     */
    public Point2D rotate(double t) {
	return new Point2D(Math.cos(t) * getX() - Math.sin(t) * getY(),
			   Math.sin(t) * getX() + Math.cos(t) * getY());
    }

    /**
     * @return this * s
     */
    public Point2D multiply(double t) {
	return new Point2D(getX() * t, getY() * t);
    }

    /**
     * Compute the dot product of this and the given point.
     *
     * @param p2 the point to use in the dot product
     * @return  the dot product.
     */
    public double dot(Point2D p2) {
	return getX()*p2.getX() + getY()*p2.getY();
    }

    /**
     * Normalize the vector and return the length.
     * 
     * @see length
     * @return the length of the vector before normalization.
     */
    public double normalize() {
	double l = Math.sqrt(distanceSq(0.0, 0.0));
	setLocation(getX()/l, getY()/l);
	return l;
    }

    /**
     * @return the length of the vector.
     */
    public double length() {
	return distance(0.0, 0.0);
    }

    /**
     * Compute average of two points.
     *
     * @return the average of the given points.
     */
    public static Point2D average(Point2D p1, Point2D p2) {
	return (p1.add(p2)).scale(0.5);
    }
}
