/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

import java.awt.*;
import java.awt.geom.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.border.*;
import javax.swing.JMenuBar;
import javax.swing.JTabbedPane;
import javax.imageio.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.util.*;
import javax.swing.event.*;
import java.awt.image.*;
import java.net.*;

/**
 * The SingleFrame is an object to represent the interface for
 * the single applet.
 */
class SingleFrame { //extends JFrame{
    ///public static int canvasWidth = 256;
    ///public static int canvasHeight = 256;

    public static int safeInt(String str){
	try {
	    return Integer.valueOf(str);
	}
	catch(NumberFormatException nfe){
	    return 0;
	}
    }

    class IntegerTextField extends JTextField {	
	JSlider partner;
	
	IntegerTextField(int val){
	    partner = null;
	    setText(val);
	}
	IntegerTextField(JSlider slider){
	    super("0", 4);
	    partner = slider;
	}
	public void processKeyEvent(KeyEvent ev){    
	    char c = ev.getKeyChar();
	    
	    if((c>='0' && c<='9') || ev.getKeyCode()==KeyEvent.VK_BACK_SPACE){
		super.processKeyEvent(ev);
		if(partner != null && getText().length()>0){
		    partner.setValue(SingleFrame.safeInt(getText()));
		    ChangeListener listeners[] = partner.getChangeListeners();
		    for(int i=0; i<listeners.length; i++)
			listeners[i].stateChanged(new ChangeEvent(partner));
		}
	    }
	    else {
		ev.consume();
	    }
	}
	public void setText(int val){
	    super.setText(String.valueOf(val));
	}
	public int getValue(){
	    return SingleFrame.safeInt(getText());
	}
    }

    class JTextSlider extends JPanel {
	IntegerTextField text;
	JSlider slider;
	boolean textMode;
	JTextSlider(int orient, int min, int max, int value){
	    slider = new JSlider(orient, min, max, value);
	    text = new IntegerTextField(this.slider);
	    textMode = true;
	    setTextMode(false);

	    slider.setValue(value);
	    //add(text);
	}
	public void setTextMode(boolean textModeNew){
	    if(textMode==textModeNew)return;

	    if(textModeNew)
		text.setText(slider.getValue());
	    else 
		slider.setValue(SingleFrame.safeInt(text.getText()));
	    this.textMode = textModeNew;

	    slider.setVisible(!textMode);
	    text.setVisible(textMode);
	    
	    removeAll();
	    //text.setMinimumSize(getMinimumSize());
	    
	    
	    //slider.setMinimumSize(getMinimumSize());
	    //slider.setPreferredSize(getPreferredSize());
	    
	    BoxLayout boxLayout = new BoxLayout(this, BoxLayout.X_AXIS);
	    setLayout(boxLayout);

	    if(textMode)add(text);
	    else add(slider);

	    validate();
	    
	}
	/*public void paint(Graphics g){
	    if(textMode){
		System.out.println("Drawing text\n");
		text.setSize(getSize());
		text.paint(g);
		return;
	    }
	    super.paint(g);
	}
	*/
	/*	public Dimension getMinimumSize(){
	    return text.getMinimumSize();
	    }*/
	public void setValue(int v){
	    text.setText(String.valueOf(v));
	    slider.setValue(v);
	}
	public int getValue(){
	    if(textMode)return (int)SingleFrame.safeInt(text.getText());
	    return slider.getValue();
	}
	public void addChangeListener(ChangeListener ce){
	    slider.addChangeListener(ce);
	}
    };
    JDialog saveStatsDialog = null; //Only want one of these guys.
    IntegerTextField canvasWidth, canvasHeight;
    
    JTextSlider seed;
    Polygon  pgon;
    JTextSlider thickness;
          
    boolean showStats;
    boolean showControl;
    Vector<String> stats;

    //JSlider randomProb, circleProb, squareProb, rectProb, starProb;
    //JSlider starMethod, pointMethod, subdivMethod;
    JTextSlider psmooth, smooth, convex, npieces, nsides;
    
    JButton create, same, fillColor, lineColor;
    JPanel  panel;
    
    JCheckBox opaque;
    JCheckBox textEntry;

    DropTarget dropTarget;

    

    class ShapeAndParam {
	String params;
	Shape  shape;
	public ShapeAndParam(Shape s, String p){
	    params = p;
	    shape = s;
	}
	public Shape getShape(){
	    return shape;
	}
	public String getParamString(){
	    return params;
	}
    }
    class ShapeCellRenderer extends JLabel implements ListCellRenderer {
	Shape activeShape;
	ShapeCellRenderer(){
	    activeShape = null;
	}
	public void paint(Graphics g){
	    super.paint(g);
	    Graphics2D g2 = (Graphics2D)g;

	    double maxx=0, maxy=0;
	    int w = canvasWidth.getValue();
	    int h = canvasHeight.getValue();

	    maxx = w;
	    maxy = h;

	    double sc = Math.min(((double)getWidth())/maxx,
				 ((double)getHeight())/maxy);
	    g2.scale(sc, sc);
	    
	    g2.setPaint(new Color(1.f,1.f,1.f,0.3f));
	    g2.fillRect(0,0,w,h);
	    g2.drawRect(0,0,w,h);

	    //for(int i=0; i<pgons.size(); i++)
	    //pgons.elementAt(i).draw(g2);
	    if(activeShape != null)
		activeShape.draw(g2);
	}
	public Component getListCellRendererComponent(
						      JList list,
						      Object value,            // value to display
						      int index,               // cell index
						      boolean isSelected,      // is the cell selected
						      boolean cellHasFocus)    // the list and the cell have the focus
	{
	    activeShape = ((ShapeAndParam) value).getShape();

	    if (isSelected) {
		setBackground(list.getSelectionBackground());
		setForeground(list.getSelectionForeground());
	    }
	    else {
		setBackground(list.getBackground());
		setForeground(list.getForeground());
	    }
	    setEnabled(list.isEnabled());
	    setFont(list.getFont());
	    setOpaque(true);
	    return this;
	}
    }


    DefaultListModel history;

    class Listener implements DragSourceListener{
	public void dragDropEnd(DragSourceDropEvent dsde){
	    
	}
	public void dragEnter(DragSourceDragEvent e){
	    DragSourceContext context = e.getDragSourceContext();
	    int myaction = e.getDropAction();
	    if( (myaction & DnDConstants.ACTION_COPY) != 0) { 
		context.setCursor(DragSource.DefaultCopyDrop);    
	    } 
	    else if ((myaction & DnDConstants.ACTION_MOVE) !=0){
		context.setCursor(DragSource.DefaultMoveDrop);
	    }
	    else {
		context.setCursor(DragSource.DefaultCopyNoDrop);        
	    }
	}
	public void dragExit(DragSourceEvent dse){
	    System.out.println("Drag Exit");
	}
	
	public void dragOver(DragSourceDragEvent dse){
	    System.out.println("Drag Over");
	}
	public void dropActionChanged(DragSourceDragEvent dse){
	    System.out.println("Drop Action Changed\n");
	}
    };

    class MyDropTarget implements DropTargetListener{
	public void dragEnter(DropTargetDragEvent dtde){

	}
	public void 	drop(DropTargetDropEvent dtde){
	    System.out.println("Drop\n");
	    DataFlavor flavs[] = dtde.getCurrentDataFlavors();
	    for(int i=0; i<flavs.length; i++){
		System.out.println(flavs[i]);
		System.out.println(flavs[i].getMimeType());
		if(flavs[i].getMimeType()=="text/plain"){
		    System.out.println("got text");
		}
	    
	    }
	    dtde.acceptDrop(DnDConstants.ACTION_COPY);
	    try{
		history.addElement(new ShapeAndParam(pgon, getParamString()));
		
		Transferable trans = dtde.getTransferable();

		Object o = trans.getTransferData(new DataFlavor("text/plain; class=java.lang.String; charset=Unicode", "java.lang.String"));
		System.out.println("o is :");
		System.out.println(o);

		setParamsFromString((String)o);
	    }
	    catch(Exception e){
		try{
		    System.out.println("the data type");
		    System.out.println(new DataFlavor("text/plain", "java.lang.String"));
		    
		}
		catch(Exception ee){

		}
		System.out.println("No text/plain");
		System.out.println(e);
	    }
	}
	public void dragExit(DropTargetEvent dte){
	    
	}	
	public void dragOver(DropTargetDragEvent dtde){
	    //System.out.println("Drag over\n");
	}
	public void dropActionChanged(DropTargetDragEvent dtde){

	}
    };

    class ShapePanel extends BasicShapePanel {
	public ShapePanel(){ }
	public void paint(Graphics g){
	    super.paint(g);

	    Graphics2D g2 = (Graphics2D)g;

	    double maxx=0, maxy=0;
	    int w = canvasWidth.getValue();
	    int h = canvasHeight.getValue();

	    maxx = w;
	    maxy = h;

	    double sc = Math.min(((double)getWidth())/maxx,
				 ((double)getHeight())/maxy);
	    g2.scale(sc, sc);
	    
	    g2.setPaint(new Color(1.f,1.f,1.f,0.3f));
	    g2.fillRect(0,0,w,h);
	    g2.drawRect(0,0,w,h);

	    //for(int i=0; i<pgons.size(); i++)
	    //pgons.elementAt(i).draw(g2);
	    pgon.draw(g2);
	    
	    if(showControl){
		pgon.drawControlPoints(g2);
	    }

	    
	    if(showStats){

		g2.scale(1.0/sc, 1.0/sc);

		g2.setPaint(new Color(1.f, 1.f, 1.f, 0.3f));
		g2.fillRect(0,0,getWidth(), 24 + 12*stats.size());

		g2.setColor(Color.GRAY);	    

		String methodStrings[] = new String[4];
		methodStrings[0] = new String("Star");
		methodStrings[1] = "Random points";
		methodStrings[2] = "Subdivision"; 
		methodStrings[3] = "Yardy"; 
		g2.drawString("Method: " + methodStrings[seed.getValue()%4], 0, 12);
		
		for(int i=0; i<stats.size(); i++)
		    g2.drawString(stats.elementAt(i), 0, 24 + 12*i);
	    }
	}
	public int getCanvasWidth(){
	    return canvasWidth.getValue();
	}
	public int getCanvasHeight(){
	    return canvasHeight.getValue();
	}
	public Vector<Shape> getShapes(){
	    Vector<Shape> v = new Vector<Shape>();
	    v.addElement(pgon);
	    return v;
	}
    }

    class Updater implements ChangeListener, ActionListener{
	public void actionPerformed(ActionEvent ae){
	    updateShape();
	}
	public void stateChanged(ChangeEvent ce){
	    updateShape();
	}
    };

    String sliderParamString(String s, JTextSlider slider){
	return s + ":" + String.valueOf(slider.getValue());
    }

    String sliderParamString(String s, JSlider slider){
	return s + ":" + String.valueOf(slider.getValue());
    }

    Color loadColor(String s){
	int i = Integer.parseInt(s, 16);
	int red = i/65536;
	int green = (i-red*65536)/256;
	int blue =  i%256;
	return new Color(red, green, blue);
    }
    void setParamsFromString(String str){
	String pvalues[] = str.split(";");
	for(int i=0; i<pvalues.length; i++){
	    String pair[] = pvalues[i].split(":");
	    if(pair.length!=2)continue;

	    System.out.println(pvalues[i]);
	    System.out.println(pair[0] + ", " + pair[1]);
		    
	    if(pair[0].compareTo("seed")==0){
		seed.setValue(SingleFrame.safeInt(pair[1]));
	    }
	    else if(pair[0].compareTo("#sides")==0){
		nsides.setValue(Integer.valueOf(pair[1]));
	    }
	    else if(pair[0].compareTo("#piece")==0){
		npieces.setValue(Integer.valueOf(pair[1]));
	    }
	    else if(pair[0].compareTo("convex")==0){
		convex.setValue(Integer.valueOf(pair[1]));
	    }
	    else if(pair[0].compareTo("smooth")==0){
		smooth.setValue(Integer.valueOf(pair[1]));
	    }
	    else if(pair[0].compareTo("%smooth")==0){
		psmooth.setValue(Integer.valueOf(pair[1]));
	    }
	    else if(pair[0].compareTo("lineColor")==0){
		lineColor.setBackground(loadColor(pair[1]));
	    }
	    else if(pair[0].compareTo("fillColor")==0){
		fillColor.setBackground(loadColor(pair[1]));
	    }
	    else if(pair[0].compareTo("line")==0){
		thickness.setValue(Integer.valueOf(pair[1]));
	    }
	    else if(pair[0].compareTo("opaque")==0){
		opaque.setSelected(Integer.valueOf(pair[1]) != 0);
	    }
	}
	updateShape();
    }
    String colorString(String s, Color c){
	return s + ":" + Integer.toHexString((c.getRed()*65536 + c.getGreen()*256 + c.getBlue()));
    }
    String getParamString(){
	String s = new String("");
	s += "seed:" + String.valueOf(seed.getValue()) + ";";
	s += sliderParamString("#sides", nsides) +";";
	s += sliderParamString("#piece", npieces) +";";
	s += sliderParamString("convex", convex) +";";
	s += sliderParamString("smooth", smooth) +";";
	s += sliderParamString("%smooth", psmooth) +";";
	s += sliderParamString("line", thickness) +";";
	s += "opaque:" + String.valueOf(opaque.isSelected()?1:0) + ";";
	s += colorString("fillColor", fillColor.getBackground()) + ";";
	s += colorString("lineColor", lineColor.getBackground());
	return s + ";\n";
    }

    void setupMenuDrag(JPanel panel){
	this.dragSource = DragSource.getDefaultDragSource();
	this.dsListener = new Listener();
	this.dgListener = new DragGestureListener(){
		public void dragGestureRecognized(DragGestureEvent dge){
		    System.out.println("DragGesture Recognized\n");
			
		    // check to see if action is OK ...
		    try {
			Transferable transferable = 
			    new ParamTransferable(getParamString());

			//initial cursor, transferable, dsource listener      
			dragSource.startDrag(dge, DragSource.DefaultCopyNoDrop, transferable, dsListener);
			    
		    }catch( InvalidDnDOperationException idoe ) {
			System.err.println( idoe );
		    }
		}
	    };
	this.dragSource.createDefaultDragGestureRecognizer(
							   panel, DnDConstants.ACTION_COPY_OR_MOVE, this.dgListener );
    }
    class ImgFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File file){
	    String str = file.getName();
	    int dotpos = str.length()-4;
	    return (str.indexOf(".txt") == dotpos ||
		    str.indexOf(".bmp") == dotpos);
		//str.indexOf(".tiff") == (dotpos-1));
	}
	public String getDescription(){
	    return "Image files (*.tiff, *.jpg, *.bmp)";
	}
    }

    class TxtFilter extends javax.swing.filechooser.FileFilter {
	public TxtFilter(){ }
	public boolean accept(File file){
	    String str = file.getName();
	    return str.indexOf(".txt") == str.length()-4;
	}
	public String getDescription(){
	    return "Parameter text file (*.txt)";
	}
    }

    class CsvFilter extends javax.swing.filechooser.FileFilter {
	public CsvFilter(){ }
	public boolean accept(File file){
	    String str = file.getName();
	    return str.indexOf(".csv") == str.length()-4;
	}
	public String getDescription(){
	    return "Comma separated values (*.csv)";
	}
    }

    public SingleFrame(final Container cont, JMenuBar menuBar){
	JMenu fmenu = new JMenu("File"), optionsMenu = new JMenu("Options"),
	    historyMenu = new JMenu("History");
	JMenuItem openItem, saveItem, saveImageItem, addCurrentItem, clearItem;
	JMenuItem saveImageSeq;
	JMenuItem saveStatsItem;

	final JMenuItem showStatsItem;
	final JMenuItem showControlItem;
	fmenu.add(openItem = new JMenuItem("Open"));
	fmenu.add(saveItem = new JMenuItem("Save"));
	fmenu.add(saveImageItem = new JMenuItem("Save Image"));
	fmenu.add(saveImageSeq = new JMenuItem("Image Sequence"));
	fmenu.add(saveStatsItem = new JMenuItem("Save Stats"));

	openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.ALT_MASK));
	saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
	saveImageItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.ALT_MASK));
	fmenu.setMnemonic(KeyEvent.VK_F);
	menuBar.add(fmenu);

	showStats = true;
	optionsMenu.add(showStatsItem = new JCheckBoxMenuItem("Show Stats"));
	showStatsItem.setSelected(showStats);
	showStatsItem.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    showStats = showStatsItem.isSelected();
		    panel.repaint();
		}
	    });
	
	optionsMenu.add(showControlItem = new JCheckBoxMenuItem("Control Points"));
	showControlItem.setSelected(showControl = false);
	showControlItem.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    showControl = showControlItem.isSelected();
		    panel.repaint();
		}
	    });
	
	optionsMenu.setMnemonic(KeyEvent.VK_O);
	menuBar.add(optionsMenu);

	historyMenu.add(addCurrentItem = new JMenuItem("Store"));
	historyMenu.addSeparator();
	historyMenu.add(clearItem = new JMenuItem("Clear"));
	
	addCurrentItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.ALT_MASK));
	clearItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, ActionEvent.ALT_MASK));
				      
	historyMenu.setMnemonic(KeyEvent.VK_H);
	menuBar.add(historyMenu);

	openItem.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    JFileChooser chooser = new JFileChooser();
		    chooser.setFileFilter(new TxtFilter());
		    chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		    int ret = chooser.showOpenDialog(null);
		    
		    if(ret == JFileChooser.APPROVE_OPTION){
			try {
			    File file = chooser.getSelectedFile();
			    BufferedReader reader = new BufferedReader(new FileReader(file));
			    String string = null;
			    String firstString = null;
			    boolean      first = true;

			    history.clear();

			    while((string = reader.readLine()) != null){
				if(first){
				    firstString = string;
				    first = false;
				}
				else {
				    //Should be a way to get a polygon from the string, until now just use the UI to make the pgon
				    setParamsFromString(string);
				    //Use the getParamString because it adds newline
				    history.addElement(new ShapeAndParam(pgon, getParamString()));
				}
				System.out.println(string);
			    }
			    setParamsFromString(firstString);
			}
			catch(Exception e){
			    System.err.println("Caught: " + e);
			}			
		    }
		}
	    });

	saveItem.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    
		    JFileChooser chooser = new JFileChooser();
		    chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		    chooser.setFileFilter(new TxtFilter());
		    chooser.setApproveButtonText("Save");
		    chooser.setDialogTitle("Save Parameters");
		    int ret = chooser.showSaveDialog(null);
		    if(ret == JFileChooser.APPROVE_OPTION){
			System.out.println("Saving as" + chooser.getSelectedFile().getName());
						
			try {
			    String str = getParamString();
			    FileWriter writer = new FileWriter(chooser.getSelectedFile());
			    writer.write(str);
			    for(int i=0; i<history.size(); i++){
				ShapeAndParam sap = (ShapeAndParam) history.elementAt(i);
				writer.write(sap.getParamString());
			    }
			    writer.close();
			}
			catch(Exception exp){
			    System.err.println("Caught: " + exp);
			}
		    }
		}
	    });
	saveImageItem.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    JFileChooser chooser = new JFileChooser();
		    chooser.setFileFilter(new ImgFilter());
		    int ret = chooser.showSaveDialog(null);
		    if(ret == JFileChooser.APPROVE_OPTION){
			System.out.println("Saving as" + chooser.getSelectedFile().getName());

			int width = canvasWidth.getValue();
			int height = canvasHeight.getValue();
			BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			Graphics2D g = (Graphics2D) img.getGraphics();
			try
			{
			    g.setPaint(new Color(1.f,1.f,1.f,1.0f));
			    g.fillRect(0, 0, width, height);
			    
			    pgon.draw(g);
			    
			    String fileName = chooser.getSelectedFile().getName();
			    int dot = (fileName.lastIndexOf("."));
			    ImageIO.write(img, fileName.substring(dot+1, fileName.length()), new FileOutputStream(chooser.getSelectedFile()));
			}
			catch (Exception e)
			{
			    JOptionPane.showMessageDialog(null, "Error writing image file.");
			}
		    }
		}
	    });

	saveImageSeq.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    JFileChooser chooser = new JFileChooser();
		    chooser.setFileFilter(new ImgFilter());
		    int ret = chooser.showSaveDialog(null);
		    if(ret == JFileChooser.APPROVE_OPTION){
			String fileName = chooser.getSelectedFile().getAbsolutePath();
			int dot = fileName.lastIndexOf(".");
			if(dot>=0){
			    JLabel label;
			    //ProgressMonitor mon = new ProgressMonitor(panel, "Progress", "On image", 0,100);
			    //mon.setMillisToPopup(500);
			    //mon.show();

			    int width = canvasWidth.getValue();
			    int height = canvasHeight.getValue();
				    
			    String fileBase = fileName.substring(0, dot);
			    String ext = fileName.substring(dot+1, fileName.length());

			    
			    int confirm = JOptionPane.showConfirmDialog(cont, 
								    "Files will be written to "+ fileBase + "-0." + ext +".  Is this okay?");
			    
			    if(confirm == JOptionPane.YES_OPTION){
				Cursor curs = cont.getCursor();
				cont.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				int length = history.size();
				for(int si = 0; si<=length; si++){
				    if(si>0)
					setParamsFromString(((ShapeAndParam)history.elementAt(si-1)).getParamString());
				    
				    for(int i=0; i<=100; i++){
					BufferedImage img = 
					    new BufferedImage(width, height, 
							      BufferedImage.TYPE_INT_RGB);
				    
					smooth.setValue(i);
					updateShape();
				    
					String curFileName = "";
					if(length==0)
					    curFileName = String.format(fileBase+"-%d."+ext, i);
					else {
					    curFileName = String.format(fileBase+"-%d-%d."+ext, seed.getValue(), i);
					}

					File curFile = new File(curFileName);
					if(curFile.exists()){
					    JOptionPane.showMessageDialog(null, "Current file " + curFileName +" already exists.  Please remove before continuing.");
					    break;
					}
					Graphics2D g = (Graphics2D) img.getGraphics();
					try
					{
					    g.setPaint(new Color(1.f,1.f,1.f,1.0f));
					    g.fillRect(0, 0, width, height);
					
					    pgon.draw(g);
					
					    ImageIO.write(img, ext, new FileOutputStream(curFileName));
					}
					catch (Exception e)
					{
					    JOptionPane.showMessageDialog(null, "Error writing image file, sequence writing will not continue.");
					    break;
					}
				    
					Thread.yield();
				    }
				    cont.setCursor(curs);
				}
			    }
			    //}
			    //}
			    //WriteThread thread = new WriteThread(dlg, label, dot, fileName);
			}
			else
			    JOptionPane.showMessageDialog(null, "Please enter a filename that ends with a .png or .jpg");
		    }
		}
	    });
	
	saveStatsItem.addActionListener(new ActionListener(){
		JCheckBox seedCheck, ncontCheck, lengthCheck;
		JCheckBox npointsCheck, nsmoothCheck, nobtuseCheck, nacuteCheck;
		JCheckBox minAngleCheck, maxAngleCheck, avgAngleCheck;
		JCheckBox areaCheck, areaPerimCheck, totalAngleCheck;

		public void actionPerformed(ActionEvent ae){
		    //If we already created the dialog, then reuse it
		    if(saveStatsDialog != null) {
			saveStatsDialog.show(true);
			return;
		    }
		    saveStatsDialog = new JDialog();
		    JDialog dialog = saveStatsDialog;

		    dialog.setSize(new Dimension(350, 230));
		    
		    Container container = dialog.getContentPane();
		    JPanel wholePanel = new JPanel();

		    JPanel panel = new JPanel();
		    
		    panel.setBorder(BorderFactory.createTitledBorder("Report contents"));

		    panel.setLayout(new GridLayout(6, 2));

		    panel.add(seedCheck = new JCheckBox("Seed"));
		    panel.add(ncontCheck = new JCheckBox("# Contours"));
		    panel.add(lengthCheck = new JCheckBox("Length"));
		    panel.add(npointsCheck = new JCheckBox("# Points"));
		    panel.add(nsmoothCheck = new JCheckBox("# Smooth"));
		    panel.add(nobtuseCheck = new JCheckBox("# Obtuse"));
		    panel.add(nacuteCheck = new JCheckBox("# Acute"));
		    panel.add(avgAngleCheck = new JCheckBox("Avg line angle"));
		    panel.add(minAngleCheck = new JCheckBox("Min line angle"));
		    panel.add(maxAngleCheck = new JCheckBox("Max line angle"));
		    panel.add(areaCheck = new JCheckBox("Area"));
		    panel.add(areaPerimCheck = new JCheckBox("Area/Perim"));
		    panel.add(totalAngleCheck = new JCheckBox("Total Angle"));

		    seedCheck.setSelected(true);
		    nacuteCheck.setSelected(true);
		    nobtuseCheck.setSelected(true);
		    minAngleCheck.setSelected(true);
		    maxAngleCheck.setSelected(true);
		    avgAngleCheck.setSelected(true);

		    //wholePanel.setLayout(new BoxLayout(wholePanel, BoxLayout.Y_AXIS));

		    wholePanel.add(panel);
		    
		    JButton export = new JButton();
		    wholePanel.add(export = new JButton("Export"));
		    
		    Dimension dim = wholePanel.getPreferredSize(); 
		    dialog.setSize(new Dimension((int)(dim.getWidth() + 20), 
						 (int)(dim.getHeight() + 20)));

		    export.addActionListener(new ActionListener(){
			    
			    public void actionPerformed(ActionEvent ae){
				JFileChooser chooser = new JFileChooser();
				chooser.setDialogType(JFileChooser.SAVE_DIALOG);
				chooser.setFileFilter(new CsvFilter());
				chooser.setApproveButtonText("Save");
				chooser.setDialogTitle("Export report");
				int ret = chooser.showSaveDialog(null);

				if(ret == JFileChooser.APPROVE_OPTION){
				    System.out.println("Saving as" + chooser.getSelectedFile().getName());
				    
				    try {
					FileWriter writer = new FileWriter(chooser.getSelectedFile());
					String backup = getParamString();
					   
					writer.write(getHeader());
					writer.write(getCSV(new ShapeAndParam(pgon, getParamString())));

					for(int i=0; i<history.size(); i++){
					    ShapeAndParam sap = (ShapeAndParam) history.elementAt(i);
					    writer.write(getCSV(sap));
					}
					writer.close();

					setParamsFromString(backup);
				    }
				    catch(Exception exp){
					System.err.println("Caught: " + exp);
					JOptionPane.showMessageDialog(null, "Error exporting stats to CSV file.");
				    }
				}				
			    }
			});
		    container.add(wholePanel);
		    dialog.show(true);
		}

		public String getHeader(){
		    String str = "";

		    if(seedCheck.isSelected())
			str += "Seed, ";

		    if(ncontCheck.isSelected())
			str += "# Contours, ";

		    if(lengthCheck.isSelected())
			str += "Length, ";
		    
		    if(npointsCheck.isSelected())
			str += "# Points, ";
		    
		    if(nsmoothCheck.isSelected())
			str += "# Smooth, ";
		    
		    Polygon.LineAngleStats stats = pgon.getLineAngleStats();

		    if(nobtuseCheck.isSelected())
			str += "# Obtuse, ";

		    if(nacuteCheck.isSelected())
			str += "# Acute, ";

		    if(avgAngleCheck.isSelected())
			str += "Avg line angle, ";

		    if(totalAngleCheck.isSelected())
			str += "Total Angle, ";

		    if(minAngleCheck.isSelected())
			str += "Min line angle, ";

		    if(maxAngleCheck.isSelected())
			str += "Max line angle, ";

		    if(areaCheck.isSelected())
			str += "Area, ";
		    
		    if(areaPerimCheck.isSelected())
			str += "Area/Perim, ";
		    
		    return str + "\n";
		}

		public String getCSV(ShapeAndParam s){
		    Polygon pgon = (Polygon) s.getShape();
		    String str = "";
		    //No way to get seed without setting from parameters.
		    setParamsFromString(s.getParamString());
		    
		    if(seedCheck.isSelected())
			str += seed.getValue() + ",";

		    if(ncontCheck.isSelected())
			str += pgon.getNumberOfContours() + ",";

		    if(lengthCheck.isSelected())
			str += pgon.getLength() + ",";
		    
		    if(npointsCheck.isSelected())
			str += pgon.getNumberOfCorners() + ",";
		    
		    if(nsmoothCheck.isSelected())
			str += pgon.getNumberOfSmoothCorners() + ",";
		    
		    Polygon.LineAngleStats stats = pgon.getLineAngleStats();

		    if(nobtuseCheck.isSelected())
			str += stats.nobtuse + ",";

		    if(nacuteCheck.isSelected())
			str += stats.nacute + ",";

		    if(avgAngleCheck.isSelected())
			str += stats.avg*180.0/Math.PI + ",";

		    if(totalAngleCheck.isSelected())
			str += stats.getTotalAngle()*180.0/Math.PI + ",";
		    
		    if(minAngleCheck.isSelected())
			str += stats.min*180.0/Math.PI + ",";

		    if(maxAngleCheck.isSelected())
			str += stats.max*180.0/Math.PI + ",";
		    
		    double a = pgon.getArea(canvasWidth.getValue(), canvasHeight.getValue());
		    if(areaCheck.isSelected())
			str += a + ",";
		    
		    if(areaPerimCheck.isSelected())
			str += a/pgon.getLength() + ",";
		    

		    return str + "\n";
		}
	    });

	addCurrentItem.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    history.addElement(new ShapeAndParam(pgon, getParamString()));
		}
	    });
	clearItem.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    history.clear();
		}
	    });

	JPanel menu = new JPanel();
	panel = new ShapePanel();
	
	dropTarget = new DropTarget(menu, new MyDropTarget());
	setupMenuDrag(menu);

	history = new DefaultListModel();//new Vector<ShapeAndParam>();
	final JList historyList = new JList(history);
	historyList.setFixedCellWidth(64);
	historyList.setFixedCellHeight(64);
	historyList.setCellRenderer(new ShapeCellRenderer());
	historyList.setSize(new Dimension(64, 100));
	historyList.setMaximumSize(new Dimension(84, 10000));
	historyList.addListSelectionListener(new ListSelectionListener(){
		public void valueChanged(ListSelectionEvent sel){
		    int selected = historyList.getSelectedIndex();
		    if(selected>=0){
			setParamsFromString(((ShapeAndParam)history.elementAt(selected)).getParamString());
		    }
		}
	    });
	
	Container topBottom = new JPanel();
	topBottom.setLayout(new BoxLayout(topBottom, BoxLayout.X_AXIS));
	topBottom.add(panel);
	JScrollPane scroll = new JScrollPane(historyList);
	scroll.setMaximumSize(new Dimension(120, 10000));
	scroll.setSize(new Dimension(120, cont.getHeight()));
	topBottom.add(scroll);



	JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, menu, topBottom);

	seed = new JTextSlider(JSlider.HORIZONTAL,  0, 0x7FFFFFFF, 1);
	convex = new JTextSlider(JSlider.HORIZONTAL, 0, 100, 25);
	smooth = new JTextSlider(JSlider.HORIZONTAL, 0, 100, 50);
	psmooth = new JTextSlider(JSlider.HORIZONTAL, 0, 100, 50);
	nsides = new JTextSlider(JSlider.HORIZONTAL, 3, 20, 7);
	npieces = new JTextSlider(JSlider.HORIZONTAL, 1, 6, 1);
	
	thickness = new JTextSlider(JSlider.HORIZONTAL, 0, 15, 9);
	create = new JButton("Next");
	same   = new JButton("Same");
	opaque = new JCheckBox("Opaque");
	textEntry = new JCheckBox("Text Mode");
	
	fillColor = new JButton("Fill");
	lineColor = new JButton("Line");

	fillColor.setPreferredSize(new Dimension(90, (int)fillColor.getPreferredSize().getHeight()));
	fillColor.setBackground(Color.WHITE);
	fillColor.setMinimumSize(fillColor.getPreferredSize());
	fillColor.setSize(fillColor.getPreferredSize());
	
	lineColor.setPreferredSize(fillColor.getPreferredSize());
	lineColor.setBackground(Color.BLACK);
	lineColor.setSize(fillColor.getPreferredSize());
			  

	opaque.setSelected(true);
	textEntry.setSelected(false);

	JPanel tab = menu;

	tab.setLayout(new BoxLayout(tab, BoxLayout.Y_AXIS));
	tab.add(textEntry);

	tab.add(makeRow("   size", canvasWidth = new IntegerTextField(256), canvasHeight = new IntegerTextField(256)));
	tab.add(makeRow("   seed", seed));
	tab.add(makeRow(" #sides",nsides));
	tab.add(makeRow(" #piece",npieces));
	tab.add(makeRow( "convex",convex));
	tab.add(makeRow(" smooth",smooth));
	tab.add(makeRow("%smooth",psmooth));
	tab.add(makeRow("line sz", thickness));
	tab.add(fillColor);
	tab.add(lineColor);//makeRow(colorButton, new JButton("Line")));//makeRow("   fill", colorButton));
	tab.add(opaque);
	tab.add(makeRow(create, same));
	
	addSpace(tab);

	splitPane.setDividerLocation(160);
	
	cont.add(splitPane);
	//cont.add(panel);

	same.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    int sd = seed.getValue();
		    seed.setValue(sd + 4);
		    updateShape();
		}
	    });
	create.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    int sd = seed.getValue();
		    seed.setValue(sd + 1);
		    updateShape();
		}
	    });
	
	fillColor.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    final JColorChooser ch = new JColorChooser(fillColor.getBackground());
		    JDialog dialog = 
			JColorChooser.createDialog(null, "Choose", true, 
						   ch, 
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						       fillColor.setBackground(ch.getColor());
						       updateShape();
						   }
					       }
					       ,
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						   }
					       });
		    dialog.show(true);
		}
	    });
	lineColor.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    final JColorChooser ch = new JColorChooser(lineColor.getBackground());
		    JDialog dialog = 
			JColorChooser.createDialog(null, "Choose", true, 
					       ch, 
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						       lineColor.setBackground(ch.getColor());
						       updateShape();
						   }
					       }
					       ,
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						   }
					       });
		    dialog.show(true);
		}
	    });

	Updater updater = new Updater();
	seed.addChangeListener(updater);
	nsides.addChangeListener(updater);
	convex.addChangeListener(updater);
	npieces.addChangeListener(updater);
	smooth.addChangeListener(updater);
	psmooth.addChangeListener(updater);
	thickness.addChangeListener(updater);
	opaque.addActionListener(updater);
	textEntry.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    seed.setTextMode(textEntry.isSelected());
		    nsides.setTextMode(textEntry.isSelected());
		    convex.setTextMode(textEntry.isSelected());
		    npieces.setTextMode(textEntry.isSelected());
		    smooth.setTextMode(textEntry.isSelected());
		    psmooth.setTextMode(textEntry.isSelected());
		    thickness.setTextMode(textEntry.isSelected());
		    panel.repaint();
		}
	    });

	updateShape();

	/*
	JFrame frame = new JFrame();
	
	frame.getContentPane().add(menu);
	frame.setSize(new Dimension(320, 240));
	frame.setVisible(true);
	*/
    }

    private void updateShape(){
	int sd;
	Random.setSeed(sd = seed.getValue());//Integer.valueOf(seed.getText()));

	pgon = new Polygon();
	double array [] = {((sd%4)==0)?1.0:0.0, ((sd%4)==1)?1.0:0.0, ((sd%4)==2)?1.0:0.0, ((sd%4)==0)?1.0:0.0};
	double convexUse = ((double)convex.getValue())/100.0;
	pgon.create(array, nsides.getValue(), ((double)smooth.getValue())/100.0, 
		    (convexUse==1.0), npieces.getValue(), ((double)psmooth.getValue())/100.0, convexUse);

	pgon.center();
	pgon.scale(0.9);
	pgon.add(new Point2D(1,1));
	pgon.scale(canvasWidth.getValue()/2.0);
	
	pgon.setThickness(thickness.getValue());
	pgon.setFill(opaque.isSelected());

	pgon.color = fillColor.getBackground();
	pgon.lineColor = lineColor.getBackground();

	panel.repaint();


	//Need to update any stats.
	stats = new Vector<String>();
	
	int width = canvasWidth.getValue();
	int height = canvasHeight.getValue();

	double area = pgon.getArea(width, height);
	double length = pgon.getLength();
	double areaperim = 0;

	areaperim = area/length;
	area = (double)((int)(area * 1000))/(1000.0);
	length = (double)((int)(length * 1000))/(1000.0);
	areaperim = (double)((int)(areaperim * 1000))/(1000.0);
	
	stats.addElement("Area      : " + area);
	stats.addElement("Perimeter : " + length);
	stats.addElement("Area/Perim: " + areaperim);
	stats.addElement("# Contours: " + pgon.getNumberOfContours());
	stats.addElement("Smooth    : " + pgon.getNumberOfSmoothCorners() + "/" + pgon.getNumberOfCorners());
    }
    
    private void addSpace(Container p){
	JPanel space = new JPanel();
	space.setPreferredSize(new Dimension(10,1000));
	p.add(space);
    }

    private Container makeRow(String label, JComponent slider){
	JLabel l = new JLabel(label);
	l.setMinimumSize(new Dimension(70, 6 + (int)l.getMinimumSize().getHeight()));
	l.setPreferredSize(l.getMinimumSize());
	l.setHorizontalTextPosition(SwingConstants.RIGHT);
	slider.setMinimumSize(new Dimension(80,(int)slider.getMinimumSize().getHeight()));

	return makeRow(l, slider);
    }

    private Container makeRow(JComponent comp, JComponent slider){
	JPanel row = new JPanel();
	
	row.add(comp);
	row.add(slider);
	row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));
	return row;
    }

    private Container makeRow(String s, JComponent v1, JComponent v2){
	JPanel row = new JPanel();
	JLabel l = new JLabel(s);
	l.setMinimumSize(new Dimension(70, 6 + (int)l.getMinimumSize().getHeight()));
	l.setPreferredSize(l.getMinimumSize());
	l.setHorizontalTextPosition(SwingConstants.RIGHT);
	row.add(l);
	row.add(v1);
	row.add(v2);
	row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));
	return row;
    }

    private DragSource dragSource;
    private DragGestureListener dgListener;
    private Listener dsListener;
}